<?php

/**
 * @file
 * Drush implementation for the master module.
 */

/**
 * Implements hook_drush_command().
 */
function master_drush_command() {
  return array(
    'master' => array(
      'description' => 'Shows general information about master.',
      'examples' => array(
        'drush help master-status' => 'More information about the drush master-status command.',
        'drush help master-ensure-modules' => 'More information about the drush master-ensure-modules command.',
        'drush help master-export' => 'More information about drush master-export.'
      ),
    ),
    'master-ensure-modules' => array(
      'description' => 'Ensure (only) master modules are installed with all dependencies.',
      'options' => array(
        'scope' => 'Indicator for a more specific set, e.g. used to set stage specific modules',
        'skip-scope-validation' => 'Skip check on scope specific variable. By default scopes can only be used if there is at least an empty configuration set.',
        'no-disable' => 'Set if no modules shall be disabled.',
        'no-uninstall' => 'Set if no module shall be uninstalled.',
      ),
      'examples' => array(
        'drush master-ensure-modules --scope=local' => 'For the local scope, it enables all master modules and requirements. Disables and uninstalls all redundant modules.',
        'drush master-ensure-modules --scope=local --no-disable --no-uninstall' => 'Ensures master modules and requirements are enabled, but does not disable or uninstall redundant modules',
      ),
    ),
    'master-status' => array(
      'description' => 'Show list of modules with their master status',
      'options' => array(
        'scope' => 'Indicator for a more specific set, e.g. used to set stage specific modules',
        'skip-scope-validation' => 'Skip check on scope specific variable. By default scopes can only be used if there is at least an empty configuration set.',
        'status' => 'Filter by modules status. Choices: master, required, missing, redundant and/or uninstall. You can use multiple comma separated values. (i.e. --status="master,required").',
        'pipe' => 'Returns a whitespace delimited list of the names of the modules without any additional information on the state.',
        'sort' => 'Sorts the module list by machine name',
      ),
      'examples' => array(
        'drush master-status --scope=local' => 'Show all information for master modules, requirements and redundant modules for the local scope.',
        'drush master-status --scope=local --status=redundant --pipe' => 'Show a pure whitespace delimited list of all redundant modules for the local scope.',
        'drush master-status --scope=local --status=master --sort' => 'Show a sorted list of master modules of the local scope.',
      ),
    ),
    'master-export' => array(
      'description' => 'Provides a configuration array for the current module state.',
      'options' => array(
        'no-profile' => 'Do not consider the profile to be a master',
        'pipe' => 'Returns a whitespace delimited list of the names of the modules without any additional information on the state.',
        'no-uninstall-blacklist' => 'When set, output will not contain the uninstall blacklist.',
        'sort' => 'Sorts the module list by machine name',
        'scopes' => 'Comma separated list of scope identifiers. For those empty scope configs are added to the output.',
      ),
      'examples' => array(
        'drush master-export --scopes=local,stage,live' => 'Returns a PHP array for the master-modules configuration you could paste to your settings.php. It also holds empty scope configuration for local, stage and live.',
        'drush master-export --pipe --sort --no-profile' => 'Returns a whitespace delimited list of master modules, sorted by machine name. The current profile will not be considered.',
      )
    )
  );
}

/**
 * Drush callback for general information.
 */
function drush_master() {

  drush_print('Use one of the given commands to work with your master configuration.');
  drush_print('');
  $rows = array(
    array(
      'Command',
      'Description',
    ),
    array(
      'drush master-status',
      'Shows the current module status',
    ),
    array(
      'drush master-ensure-modules',
      'Sets the intended module states. Enables, disables and uninstalls modules.',
    ),
    array(
      'drush master-export',
      'Get the configuration for the current module set.',
    ),
  );

  drush_print_table($rows, TRUE);

  drush_print('Use the drush help command to find out more about the specific command.');
}

/**
 * Ensures only master modules and requirements are set via drush command.
 */
function drush_master_ensure_modules() {

  $no_disable = drush_get_option('no-disable');
  $no_uninstall = drush_get_option('no-uninstall');
  $scope = drush_get_option('scope');
  $skip_scope_validation = drush_get_option('skip-scope-validation');

  $master_modules = master_get_master_modules($scope);

  // Check if master modules are set.
  if (!master_is_set($scope)) {
    drush_log('No master modules are set!', 'error');
    return;
  }
  // Check if scope is valid.
  elseif (!$skip_scope_validation && !master_validate_scope($scope)) {
    drush_log(dt('The given scope "@scope" is not valid!', array('@scope' => $scope)), 'error');
    return;
  }

  // Get status information for the given master module set.
  $status = master_get_change_status_information($master_modules);

  // If no modules have to be enabled, we just show the status.
  if (empty($status['missing'])) {
    drush_log('All master modules and requirements are enabled.', 'ok');
  }
  // Else we invoke enabling of the other modules.
  else {
    drush_print(dt('The following modules will be enabled: !modules', array('!modules' => implode(', ', $status['missing']))));
    if (drush_confirm(dt('Do you really want to continue?'))) {
      foreach ($status['missing'] as $module) {
        drush_invoke_process(drush_sitealias_get_record('@self'), 'pm-enable', array($module));
      }
    }
    else {
      drush_die('Aborting.');
    }
  }

  $blacklist = master_get_uninstall_blacklist();
  $uninstall = array_diff($status['uninstall'], $blacklist);
  $uninstall_skip = array_intersect($status['uninstall'], $blacklist);

  // Uninstall modules that are not enabled.
  if (empty($uninstall)) {
    drush_log('No modules have to be uninstalled.', 'ok');
  }
  elseif ($no_uninstall) {
    drush_log(dt('Uninstall of modules is skipped, but following modules should be uninstalled: !modules', array('!modules' => implode(', ', $uninstall))), 'notice');
  }
  else {
    drush_print(dt('The following modules will be uninstalled: !modules', array('!modules' => implode(', ', $uninstall))));

    if (drush_confirm(dt('Do you really want to continue?'))) {
      foreach ($uninstall as $module) {
        drush_invoke_process(drush_sitealias_get_record('@self'), 'pm-uninstall', array($module));
      }
    }
    else {
      drush_die('Aborting.');
    }
  }

  // Disable modules if there are some redundant ones.
  if (empty($status['redundant'])) {
    drush_log('No modules have to be disabled.', 'ok');
  }
  elseif ($no_disable) {
    drush_log(dt('Disabling of modules is skipped, but following modules should be disabled: !modules', array('!modules' => implode(', ', $status['redundant']))), 'notice');
  }
  else {
    if (empty($no_uninstall)) {
      drush_print(dt('The following modules will be disabled and uninstalled: !modules', array('!modules' => implode(', ', $status['redundant']))));
    }
    // The message for no uninstall mode has to be a little different.
    else {
      drush_print(dt('The following modules will be disabled : !modules', array('!modules' => implode(', ', $status['redundant']))));
    }

    if (drush_confirm(dt('Do you really want to continue?'))) {
      foreach ($status['redundant'] as $module) {
        // First we disable, then we uninstall.
        drush_invoke_process(drush_sitealias_get_record('@self'), 'pm-disable', array($module));

        // Uninstall directly to avoid uninstall to break if they need
        // another module for that to be enabled.
        // Skip uninstall if the module is on the blacklist.
        if (in_array($module, $blacklist)) {
          $uninstall_skip[] = $module;
        }
        elseif (empty($no_uninstall)) {
          drush_invoke_process(drush_sitealias_get_record('@self'), 'pm-uninstall', array($module));
        }
      }

      // Show notice if no uninstall is activated.
      if (!empty($no_uninstall)) {
        drush_log(dt('Uninstall of modules is skipped, so the disabled modules were not uninstalled.'), 'notice');
      }
    }
    else {
      drush_die('Aborting.');
    }
  }

  // Show notification for modules that are on the uninstall blacklist.
  if (!empty($uninstall_skip)) {
    drush_print(dt('The following modules are on the uninstall blacklist and have not been uninstalled: !modules', array('!modules' => implode(', ', $uninstall_skip))));
  }
}

/**
 * Drush callback to show master dependent module status.
 */
function drush_master_status() {

  // Getting options from drush command.
  $scope = drush_get_option('scope');
  $status_modes = drush_get_option_list('status');
  $pipe = drush_get_option('pipe');
  $sort = drush_get_option('sort');
  $skip_scope_validation = drush_get_option('skip-scope-validation');

  $master_modules = master_get_master_modules($scope);

  // Check if master modules are set.
  if (!master_is_set($scope)) {
    drush_log('No master modules are set!', 'error');
    return;
  }
  // Check if scope is valid.
  elseif (!$skip_scope_validation && !master_validate_scope($scope)) {
    drush_log(dt('The given scope "@scope" is not valid!', array('@scope' => $scope)), 'error');
    return;
  }

  $required = master_get_required_modules($master_modules);
  $status = master_get_change_status_information($master_modules);

  // Add master and required to status, so we can reuse it in the loop in a
  // generic way.
  $status['master'] = $master_modules;
  $status['required'] = $required;

  // Definition of available modes to use in the loop.
  $modes = array(
    'master' => 'Master modules',
    'required' => 'Required modules',
    'missing' => 'Missing modules',
    'redundant' => 'Redundant modules',
    'uninstall' => 'Uninstall modules',
  );

  $pipe_modules = array();
  foreach ($modes as $mode => $label) {
    if (empty($status_modes) || in_array($mode, $status_modes)) {
      if (!empty($sort)) {
        sort($status[$mode]);
      }

      if (empty($pipe)) {
        _drush_master_print_module_table($status[$mode], $label);
      }
      else {
        $pipe_modules = array_merge($pipe_modules, $status[$mode]);
      }
    }
  }

  // Print pipe if we are in pipe mode.
  if (!empty($pipe)) {
    if (!empty($sort)) {
      sort($pipe_modules);
    }
    drush_print_pipe(array_unique($pipe_modules));
  }
}

/**
 * Helper to print a table for a given module list.
 *
 * @param $modules
 * @param string $title
 */
function _drush_master_print_module_table($modules, $title = 'Modules') {
  // Lists master modules.
  $enabled = 0;
  if (!empty($modules)) {
    $rows = array();
    $module_data = master_get_module_data();
    foreach ($modules as $module) {
      if (!isset($module_data[$module])) {
        // No module data is available, so we mark it so.
        $rows[] = array(
          $module,
          dt('n/a'),
          dt('n/a'),
        );
      }
      else {
        // Get concrete status.
        if (!empty($module_data[$module]->status)) {
          $enabled++;
          $status = 'enabled';
        }
        elseif ($module_data[$module]->schema_version == -1) {
          $status = 'uninstalled';
        }
        else {
          $status = 'disabled';
        }

        $rows[] = array(
          $module,
          $module_data[$module]->info['name'],
          $status,
        );
      }
    }
  }
  drush_print(dt('@title (@enabled / @count)', array('@enabled' => $enabled, '@title' => $title, '@count' => count($modules))));
  drush_print(str_repeat('-', 60));
  if (!empty($rows)) {
    drush_print_table($rows, TRUE);
  }
  else {
    drush_print(dt('- No modules -'));
    drush_print('');
  }
}

/**
 * Drush callback to provide an export of the current module set.
 */
function drush_master_export() {

  $no_profile = drush_get_option('no-profile');
  $pipe = drush_get_option('pipe');
  $sort = drush_get_option('sort');

  $recommended_masters = master_get_master_modules_recommendation(!$no_profile);
  // Sort if we we wanted to.
  if (!empty($sort)) {
    sort($recommended_masters);
  }

  // Default output is a PHP-Array.
  if (empty($pipe)) {
    $export = format_string('$conf[\'master_modules\'] = !var;', array('!var' => var_export($recommended_masters, TRUE)));

    $scopes = drush_get_option_list('scopes');
    if (!empty($scopes)) {
      // We add an empty
      foreach ($scopes as $scope) {
        $export .= "\n";
        $export .= format_string('$conf[\'master_modules:@scope\'] = array();', array('@scope' => $scope));
      }
    }

    // Add uninstall blacklist to php output.
    $no_uninstall = drush_get_option('no-uninstall-blacklist');
    if (empty($no_uninstall)) {
      $uninstall_blacklist = master_get_uninstall_blacklist_recommendation();
      if (!empty($uninstall_blacklist)) {
        $export .= "\n";
        $export .= format_string('$conf[\'master_uninstall_blacklist\'] = !var;', array('!var' => var_export($uninstall_blacklist, TRUE)));
      }
      // Also add it for each scope.
      if (!empty($scopes)) {
        // We add an empty module list for each scope.
        foreach ($scopes as $scope) {
          $export .= "\n";
          $export .= format_string('$conf[\'master_uninstall_blacklist:@scope\'] = array();', array('@scope' => $scope));
        }
      }
    }

    drush_print($export);
  }
  else {
    drush_print_pipe(array_unique($recommended_masters));
  }
}
